#!/usr/bin/perl -w
##################################################################
# Skript um die Grundrechenarten in LateX zu setzen
# Copyright (C) 2009 Xenia Rendtel
#
# VERSION 1
# Letzte Aenderung: 04.11.2009
#
# Dieses Programm ist freie Software. Sie koennen es unter den Bedingungen der
# GNU General Public License, wie von der Free Software Foundation
# veroeffentlicht, weitergeben und/oder modifizieren, entweder gemaeß Version 2
# der Lizenz oder (nach Ihrer Option) jeder spaeteren Version.
#
# Die Veroeffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen
# von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
# Garantie der MARKTREIFE oder der VERWENDBARKEIT
# ZWECK. Details finden Sie in der GNU General Public License.
#
# Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
# Programm erhalten haben. Falls nicht, schreiben Sie an die Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
#
# Bei Fehlern schreiben Sie diese einfach an
# webmaster@rendtel.de
##################################################################
###Weitere Pakete
use POSIX qw /floor ceil/;
use Math::Trig;
use strict;
use warnings;
use FindBin;
use lib "$FindBin::Bin";

use lib '/home/xenia/Schule/Informatik-Eigen/Adam-Ries';

use hilfsfunktionen;
use adamriesfunktionen;

my (
	$kommandodatei, $texdatei, $i,

	$leer,         $eingabedateizeilen, @eingabedatei,
	@ausgabedatei, @ausgabedateizeilen, $kommandodateinummer,
	$datei,    $zeile, $xmin, $xmax, $ausrichtung, $bezeichnung, $func,
	$variable, $j,
	$schritt, $spalten, $kopf,
	$nachkommastellen, $spaltenausrichtung, $liste, $endeerreicht, $hilfszeile, $anzahlzeilen
);

$spaltenausrichtung = "";
$kopf               = 1;
$liste              = 0;

# Arrays und Layer
$eingabedateizeilen  = 0;
@eingabedatei        = ();
@ausgabedatei        = ();
@ausgabedateizeilen  = ();
$kommandodateinummer = 0;

$datei = 0;

for ( $i = 0 ; $i <= 150 ; $i++ ) {
	$ausgabedateizeilen[$i] = 0;
}

# Die Eingabedatei wird gelesen
sub leseeingabedatei {
	my $dateiname = shift(@_);
	my $zeile;
	open( my $EINGABEDATEI, $dateiname ) || die $dateiname . ": $!";
	while ( $zeile = <$EINGABEDATEI> ) {

		# Zeilenenden beseitigen, Kommentare und Leerzeilen ignorieren
		$zeile =~ s/[\r\n]//g;
		$zeile =~ s/ +/ /g;
		$zeile =~ s/ +$//g;
		$zeile =~ s/^ +//g;
		next if ( $zeile =~ /^\#/ );
		next if ( $zeile =~ /^$/ );
		if ( $zeile =~ /^lesedatei (.*)$/i ) {
			leseeingabedatei($1);
		}
		else {

			#Einzelne Zeilen in ein Array speichern
			$eingabedatei[$eingabedateizeilen] = $zeile;
			$eingabedateizeilen++;
		}
	}
	close($EINGABEDATEI);
}

# In ein array wird geschrieben
sub schreibeinarray {
	my ( $layer, $zeile ) = @_;
	$ausgabedatei[$layer][ $ausgabedateizeilen[$layer] ] = $zeile;
	$ausgabedateizeilen[$layer]++;
}

# Der Kopf der Latex-Datei wird geschrieben
sub kopfdatei {

	if ( $kopf == 1 ) {
		kommentar( 1, "Autor: X. Rendtel" );
		kommentar( 1, "Letzte Aenderung: 2009 \n" );

		latex( 1, "\\documentclass[10pt, a4paper]{article}" );
		usepackage( "utf8", "inputenc" );
		usepackage( "",     "array,longtable,colortbl,multirow,hhline,tabularx,multicol,xcolor" );

		latex( 3, "\\pagestyle{empty} \n\\begin{document} " );
	}
}

sub fussdatei {
	if ( $kopf == 1 ) {
		latex( 11, "\\end{document}" );
	}

}

while ( exists $ARGV[$kommandodateinummer] ) {
	$kommandodatei = $ARGV[$kommandodateinummer];
	$texdatei      = $kommandodatei;
	$texdatei =~ s/\.ptxt$//i;
	$texdatei = $texdatei . " . adam ";
	$kommandodateinummer++;
	$eingabedateizeilen = 0;
	leseeingabedatei($kommandodatei);
	for ( $i = 0 ; $i < $eingabedateizeilen ; $i++ ) {
		$zeile = $eingabedatei[$i];

		# Befehl " skalierung "

		if ( $zeile =~ /^mul( [-0-9\.]+),([-0-9\.]+),([-0-9\.]+),([-0-9\.]+)$/i ) {

			multipliziere( $1, $2, $3, $4 );
		}

		elsif ( $zeile =~ /^div( [-0-9\.]+),([-0-9\.]+),([-0-9\.]+),([-0-9\.]+)$/i ) {

			dividiere( $1, $2, $3, $4 );
		}

		elsif ( $zeile =~ /^addiere$/i ) {

			$anzahlzeilen = 1;
			setsumme(0);
			setdifferenz(0);
			

			$endeerreicht = 0;

			while (( !$endeerreicht )
				&& ( $i < $eingabedateizeilen - 1 ) )
			{
				$i++;
				$hilfszeile = $eingabedatei[$i];
				if ( $hilfszeile =~ /^ende$/i ) {
					$endeerreicht = 1;
					tabellenkopf( getspaltenanzahl() );
					addierenauslesen( $anzahlzeilen - 1 );

					tabellenfuss();
				}
				elsif ( $hilfszeile =~ /^([-0-9\.]+),([-0-9\.]+)$/i ) {
					addiereneinlesen( $anzahlzeilen, $1, $2 );
					$anzahlzeilen++;
				}
				else {
					printf("%% Kaputte Zeile\n");
					$endeerreicht = 1;
				}

			}
		}

		elsif ( $zeile =~ /^subtrahiere$/i ) {

			$anzahlzeilen = 1;
			setsumme(0);
			setdifferenz(0);

			$endeerreicht = 0;

			while (( !$endeerreicht )
				&& ( $i < $eingabedateizeilen - 1 ) )
			{
				$i++;
				$hilfszeile = $eingabedatei[$i];
				if ( $hilfszeile =~ /^ende$/i ) {
					$endeerreicht = 1;
					tabellenkopf( getspaltenanzahl() );
					subtrahierenauslesen( $anzahlzeilen - 1 );

					tabellenfuss();
				}
				elsif ( $hilfszeile =~ /^([-0-9\.]+),([-0-9\.]+)$/i ) {
					if ( $anzahlzeilen == 1 ) {
						subtrahiereneinlesen( $anzahlzeilen, $1, $2 );
					}
					else {
						subtrahiereneinlesen( $anzahlzeilen, $1, $2 );
					}
					$anzahlzeilen++;
				}
				else {
					printf("%% Kaputte Zeile\n");
					$endeerreicht = 1;
				}

			}
		}

		# Alle anderen Befehle sind Schrott
		else {
			printf( "%% Unbekannter Befehl '%s'\n", $zeile );
		}
	}
}

$leer = 1;

my $layer;    
for ( $layer = 0 ; $layer <= 100 ; $layer++ ) {
	for ( $i = 0 ; $i < $ausgabedateizeilen[$layer] ; $i++ ) {
		$leer = 0;
	}
}
if ( $leer == 0 ) {
	kopfdatei();
	fussdatei();

	for ( $layer = 0 ; $layer <= 100 ; $layer++ ) {
		for ( $i = 0 ; $i < $ausgabedateizeilen[$layer] ; $i++ ) {
			printf( "%s \n", $ausgabedatei[$layer][$i] );
		}
	}
}
else {
	printf("%% Die Datei ist leer \n");
}

